@@ -56,15 +56,6 @@ checkExamples <- function(exclude=NULL, include=1:length(.fct.list),
       importMethodsFrom <- exportMethods <- S3method <- function(...) NULL
   .env <- new.env()
 
-  ##  if (!require(.package, character.only=TRUE)) return(NA)
-  exportPattern <- function(p) {
-    all.pattern <- p %in% c("^[^\\.]", "^[^.]", ".") | get("all.pattern", .env)
-    if (!.ignore.all) assign("all.pattern", all.pattern, .env)
-    if (all.pattern) return(NULL)
-    stopifnot(nchar(p)==2, substr(p,1,1)=="^")
-    assign("p", c(get("p", .env), substring(p, 2)), .env)
-  }
-
   export <- function(...) {
     ## code from 'rm'
     dots <- match.call(expand.dots = FALSE)$...
@@ -96,7 +87,7 @@ checkExamples <- function(exclude=NULL, include=1:length(.fct.list),
     .files <- dir(.path, pattern="d$")
     .fct.list <- character(length(.files))
     for (i in 1:length(.files)) {
-      cat(i, .files[i], "\n")
+      cat(i, .path, .files[i], "\n")
       #if (i == 152) {cat("jumped\n"); next}
       .content <- scan(paste(.path, .files[i], sep="/") , what=character(),
                        quiet=TRUE)
@@ -108,7 +99,7 @@ checkExamples <- function(exclude=NULL, include=1:length(.fct.list),
   }
   Print(.fct.list, length(.fct.list)) #
   .include <- include
-  RFoptions(graphics.always_close_screen = TRUE) 
+  RFoptions(graphics.close_screen = TRUE, graphics.split_screen = TRUE)
   .RFopt <- RFoptions()
   .not_working_no <- .not_working <- NULL
   .included.fl <- .fct.list[.include]
@@ -124,6 +115,7 @@ checkExamples <- function(exclude=NULL, include=1:length(.fct.list),
   }
   
   for (.idx in .include) {
+    try(repeat dev.off(), silent=TRUE)
     if (.idx %in% .exclude) next
     cat("\n\n\n\n\n", .idx, " ", .package, ":", .fct.list[.idx],
         " (total=", length(.fct.list), ") \n", sep="")
@@ -141,7 +133,7 @@ checkExamples <- function(exclude=NULL, include=1:length(.fct.list),
       command <- paste("R < ", file.in, ">>", file.out)
     } else {
       if (.halt) {
-        do.call(utils::example, list(.fct.list[.idx], ask=.ask, echo=.echo))
+         do.call(utils::example, list(.fct.list[.idx], ask=.ask, echo=.echo))
       } else {
         if (is(try(do.call(utils::example, list(.fct.list[.idx], ask=.ask,
                                                 echo=.echo))), "try-error")) {
@@ -175,7 +167,7 @@ StartExample <- function(reduced = TRUE) {
 
 FinalizeExample <- function() {
   if (!interactive()) {
-    close.screen(all.screens = TRUE)    
+    close.screen(all.screens = TRUE)  
     n <- length(dev.list()) - 2 ## otherwise R CMD check complains
     ##                             for missing graphic device
     if (n > 0) {
@@ -185,7 +177,6 @@ FinalizeExample <- function() {
     }
   }
 
- # if (RFoptions()$general$print > 1) {Print(RFoptions()$general$print); print("not ok")}  Print(RFoptions()$gen)
   
   RFoptions(seed = NA, coord_system="auto", examples_reduced=FALSE)
   
@@ -229,69 +220,87 @@ compareVersions <- function(path = "~/svn/RandomFields/cran") {
   }
 }
 
+jpegORpdf <- function(graphics=NULL) {
+  GD <- getOption("device")
+  
+  ispdf <- is.logical(all.equal(GD, "pdf"))
+  isjpg <- is.logical(all.equal(GD, "jpeg"))
+  
 
+  if (!ispdf && !isjpg) {
+    ispdf <- is.function(GD) && is.logical(all.equal(args(pdf), args(GD)))
+    isjpg <- is.function(GD) && is.logical(all.equal(args(jpeg), args(GD)))
+  } 
+  
+  if (is.null(graphics)) graphics <- RFoptions()$graphics        
+  file <- graphics$file
+  if (file != "") {
+    f <- strsplit(file, "\\.")[[1]]
+    ext <- tolower(f[length(f)])
+    is_pdf <- ext == "pdf"
+    is_jpg <- ext == "jpeg" || ext == "jpg"
+    if (is_pdf || is_jpg) graphics$file <- paste(f[-length(f)], sep=".")
+   
+    if (ispdf || isjpg) {
+      if ((is_pdf && isjpg) || (is_jpg && ispdf)) stop("'extension' does not fit given device")
+    } else {
+      ispdf <- is_pdf
+      isjpg <- is_jpg
+    } 
+  }
+
+  if (ispdf) GD <- pdf
+  else if (isjpg) GD <- jpeg
+
+  graphics$ispdf <- ispdf
+  graphics$isjpg <- isjpg
+  graphics$ext <- c(".pdf", ".jpg")[isjpg + 1]
+  graphics$GD <- GD
+
+#  Print(graphics)
+
+  return(graphics)
+  
+}
 
 ScreenDevice <- function(height, width) {
-  if (height > 0  && width > 0) {
-    GD <- getOption("device")
+  graphics <- jpegORpdf()    
+  if (graphics$ispdf || graphics$isjpg) {
+    if (graphics$filenumber == 99999)
+      stop("number of pictures exceeds max. number of pictures")        
 
-    ispdf <- is.logical(all.equal(GD, "pdf"))
-    isjpg <- is.logical(all.equal(GD, "jpeg"))
-    if (ispdf) {
-      GD <- pdf
-    } else if (isjpg) {
-      GD <- jpeg
+    file <- graphics$file
+    if (file != "") {
+      if (!graphics$onefile) {
+        file <- paste(file,
+                      formatC(format="d", flag="0", graphics$filenumber,
+                              width=5),
+                      graphics$ext, sep="")
+        RFoptions(graphics.filenumber = graphics$filenumber + 1)
+      } else file <- paste(file, graphics$ext, sep="")
+      ##            Print(file, dev.list())
+      if (graphics$ispdf)
+        graphics$GD(height=height, width=width, onefile=FALSE, file=file)
+      else if (graphics$isjpg) graphics$GD(height=height, width=width,
+                                           file=file, units="in", res=graphics$resolution)
+      else stop("device not recognized by 'RandomFields'")
+      return()
     } else {
-      ispdf <- is.function(GD) && is.logical(all.equal(args(pdf), args(GD)))
-      isjpg <- is.function(GD) && is.logical(all.equal(args(jpeg), args(GD)))
+      if (graphics$isjpg) stop("'file' in 'RFoptions' not set.")
     }
-   
-#   Print(args(pdf), args(GD), all.equal(args(pdf), args(GD)),
-  #        all.equal(args(jpeg), args(GD)),
-  #        is.function(GD), RFoptions()$graphics)
-    
-    if (ispdf || isjpg) {
-      
-      ##     Print(ispdf, isjpg)
-      ##     Print(ispdf, RFoptions()$graphics        )
-      
-      graphics <- RFoptions()$graphics        
-      if (graphics$filenumber == 999)
-        stop("number of pictures exceeds max. number of pictures")        
-      file <- graphics$file
-      ext <- c(".pdf", ".jpg")[isjpg + 1]
-      
-       if (file != "") {
-        if (!graphics$onefile) {
-         file <- paste(file,
-                        formatC(format="d", flag="0", graphics$filenumber,
-                                width=3),
-                        ext, sep="")
-         RFoptions(graphics.filenumber = graphics$filenumber + 1)
-        } else file <- paste(file, ext, sep="")
-        ##            Print(file, dev.list())
-         if (ispdf) GD(height=height, width=width, onefile=FALSE, file=file)
-        else if (isjpg) GD(height=height, width=width,
-                           file=file, units="in", res=graphics$resolution)
-        else stop("device not recognized by 'RandomFields'")
-       return()
-      } else {
-        if (isjpg) stop("'file' in 'RFoptions' not set.")
-      }
-    }
-     
-    args <- names(as.list(args(GD)))
-    if (all(c("width", "height") %in% args) &&
-        ( !any(c("file", "filename") %in% args)) || ispdf) {
-      GD(height=height, width=width)        
-      ##      Print("OK", height, width, RFoptions()$graphics, par()$cex)
-     return()
-    }
-    
-   if (RFoptions()$internal$warn_aspect_ratio) {
-      RFoptions(warn_aspect_ratio = FALSE)
-      cat("The graphical device does not seem to be a standard screen device. Hence the\naspect ratio might not be correct. (This message appears only one per session.)")
+  }
+  
+  args <- names(as.list(args(graphics$GD)))    
+  if (all(c("width", "height") %in% args) &&
+      ( !any(c("file", "filename") %in% args)) || graphics$ispdf) {
+    graphics$GD(height=height, width=width)        
+    ##    Print("OK", height, width, RFoptions()$graphics, par()$cex)
+    return()
     }
+  
+  if (RFoptions()$internal$warn_aspect_ratio) {
+    RFoptions(warn_aspect_ratio = FALSE)
+    cat("The graphical device does not seem to be a standard screen device. Hence the\naspect ratio might not be correct. (This message appears only one per session.)")
   }
 }
 
@@ -303,39 +312,52 @@ ScreenDevice <- function(height, width) {
 
 ArrangeDevice <- function(graphics, figs, dh=2.8, h.outer=1.2,
                           dw = 2.5, w.outer=0.7) {
-  if (is.na(graphics$always_open_screen)) {
-      open <- interactive()
-      RFoptions(graphics.always_open_screen = open)
-  } else open <- graphics$always_open_screen
-
-  if (graphics$always_close_screen) {
-    close.screen(all.screens=TRUE)
-    if (is.finite(graphics$height) && graphics$height>0) {
-      if (length(dev.list()) > 0) dev.off() ## OK
-    }
-  }
+#  plot(1,1); return;  Print("ARRANGE", graphics)
   
-  H <-  graphics$height
+  H <- curH <- graphics$height
+  curW <- graphics$width
   if (is.finite(H) && H>0) {
-    H <- H * pmin(1, graphics$increase_upto[1] / figs[1],
-                  graphics$increase_upto[2] / figs[2])
-    DH <- H * dh / (dh + h.outer)
-    HO <- H - DH
-    curH <- figs[1] * DH + HO
-    W <- H * (dw + w.outer) / (dh + h.outer)
-    DW <- W * dw / (dw + w.outer)
-    WO <- W - DW
-    curW <-  figs[2] * DW + WO
-    if (open) ScreenDevice(height=curH, width=curW)
-    return(c(curH, curW)) 
+    if (is.na(graphics$width) || graphics$width<=0.0) {
+      H <- H * pmin(1, graphics$increase_upto[1] / figs[1],
+                    graphics$increase_upto[2] / figs[2])
+      DH <- H * dh / (dh + h.outer)
+      HO <- H - DH
+      curH <- figs[1] * DH + HO
+      W <- H * (dw + w.outer) / (dh + h.outer)
+      DW <- W * dw / (dw + w.outer)
+      WO <- W - DW
+      curW <-  figs[2] * DW + WO
+    }
+
+    graphics <- jpegORpdf(graphics)
+    jpg_pdf <- graphics$isjpg || graphics$ispdf
+    
+   if (is.na(graphics$always_open_device)) {
+      open <- interactive() || jpg_pdf
+      RFoptions(graphics.always_open_device = open)
+    } else open <- graphics$always_open_device
+
+    if (length(dev.list()) > 0 && open) {      
+      close <- graphics$always_close_device
+      if (is.na(close)) close <- jpg_pdf
+      if (close) dev.off()
+    }
+    
+    if (open) {
+      ScreenDevice(height=curH, width=curW)
+    }
+    
+  } else warning("device could not be opened as 'height' is not positive.")
+ 
+  if (graphics$split_screen) {
+    if (length(dev.list()) > 0 && any(par()$mfcol != 1)) par(mfcol=c(1,1))
   } else {
-    return(rep(NA, 2))
+    if (any(figs != 1) && length(dev.list()) > 0) par(mfcol=figs)
   }
+  return(c(curH, curW))
 }
 
 
-
-
 reverse_dependencies_with_maintainers <-
   function(packages, which = c("Depends", "Imports", "LinkingTo"),
            recursive = FALSE) {
