@@ -281,6 +281,18 @@ void do_distr_do(cov_model *cov, double *v){
   distrR(NULL, cov, v);
 }
 void range_distr(cov_model *cov, range_type *range){
+#define distr_n 5
+  int idx[distr_n] = {DISTR_DX, DISTR_PX, DISTR_QX, DISTR_RX, DISTR_ENV};
+  for (int i=0; i<distr_n; i++) {
+    int j = idx[i];
+    range->min[j] = RF_NAN;
+    range->max[j] = RF_NAN;
+    range->pmin[j] = RF_NAN;
+    range->pmax[j] = RF_NAN;
+    range->openmin[j] = false;
+    range->openmax[j] = false; 
+  }
+
   range->min[DISTR_NROW] = 1;
   range->max[DISTR_NROW] = 10;
   range->pmin[DISTR_NROW] = 1;
@@ -295,6 +307,7 @@ void range_distr(cov_model *cov, range_type *range){
   range->openmin[DISTR_NCOL] = false;
   range->openmax[DISTR_NCOL] = false; 
   
+   
   int i,
     kappas = CovList[cov->nr].kappas;
   for (i=DISTR_LAST + 1; i<kappas; i++) {
